@@ -7,20 +7,23 @@ libvcs.git
 From https://github.com/saltstack/salt (Apache License):
 
 - :py:meth:`GitRepo.remote`
-- :py:meth:`GitRepo.remote_get`
-- :py:meth:`GitRepo.remote_set`
+- :py:meth:`GitRepo.remote_get` (renamed to ``remote``)
+- :py:meth:`GitRepo.remote_set` (renamed to ``set_remote``)
 
 From pip (MIT Licnese):
 
 - :py:meth:`GitRepo.get_url_and_revision_from_pip_url` (get_url_rev)
 - :py:meth:`GitRepo.get_revision`
+- :py:meth:`GitRepo.get_git_version`
 
 """
 from __future__ import absolute_import, print_function, unicode_literals
 
+import collections
 import logging
 import os
 import re
+import warnings
 
 from . import exc
 from ._compat import urlparse
@@ -28,50 +31,110 @@ from .base import BaseRepo
 
 logger = logging.getLogger(__name__)
 
+GitRemote = collections.namedtuple('GitRemote', ['name', 'fetch_url', 'push_url'])
+"""Structure containing git repo information.
+
+Supports :meth:`collections.namedtuple._asdict()`
+
+.. versionadded:: 0.4.0
+"""
+
+
+def extract_status(value):
+    """Returns ``git status -sb --porcelain=2`` extracted to a dict
+
+    Returns
+    -------
+    dict : 
+        Dictionary of git repo's status
+    """
+    pattern = re.compile(
+        r"""[\n\r]?
+        (
+            #
+            \W+
+            branch.oid\W+
+            (?P<branch_oid>
+                [a-f0-9]{40}
+            )
+        )?
+        (
+            #
+            \W+
+            branch.head
+            [\W]+
+            (?P<branch_head>
+                [\w-]*
+            )
+            
+        )?
+        (
+            #
+            \W+
+            branch.upstream
+            [\W]+
+            (?P<branch_upstream>
+                [/\w-]*
+            )
+        )?
+        (
+            #
+            \W+
+            branch.ab
+            [\W]+
+            (?P<branch_ab>
+                \+(?P<branch_ahead>\d+)
+                \W{1}
+                \-(?P<branch_behind>\d+)
+            )
+        )?
+        """,
+        re.VERBOSE | re.MULTILINE,
+    )
+    matches = pattern.search(value)
+    return matches.groupdict()
+
 
 class GitRepo(BaseRepo):
     bin_name = 'git'
     schemes = ('git', 'git+http', 'git+https', 'git+ssh', 'git+git', 'git+file')
 
-    def __init__(self, url, remotes=None, **kwargs):
+    def __init__(self, url, **kwargs):
         """A git repository.
 
         :param url: URL of repo
         :type url: str
 
-        :param remotes: list of remotes in dict format::
-
-            [{
-            "remote_name": "myremote",
-            "url": "https://github.com/tony/vim-config.git"
-            }]
-        :type remotes: list
-
-        :param git_remote_name: name of the remote (default "origin")
-        :type git_remote_name: str
-
         :param git_shallow: clone with ``--depth 1`` (default False)
         :type git_shallow: bool
 
-        :param git_submodules: Git submodules that shall be updated, all if
-            empty
+        :param git_submodules: Git submodules that shall be updated, all if empty
         :type git_submodules: list
 
-        :param tls_verify: Should certificate for https be checked (default
-            False)
+        :param tls_verify: Should certificate for https be checked (default False)
         :type tls_verify: bool
+
+        .. versionchanged:: 0.4.0
+
+           The ``remotes`` argument is ignored. Use :meth:`~.set_remote` to set remotes
+           before running :meth:`~.obtain`.
+
+           The ``remotes`` argument is deprecated and will be removed in 0.5
         """
-        if 'git_remote_name' not in kwargs:
-            self.git_remote_name = "origin"
         if 'git_shallow' not in kwargs:
             self.git_shallow = False
         if 'git_submodules' not in kwargs:
             self.git_submodules = []
         if 'tls_verify' not in kwargs:
             self.tls_verify = False
-        BaseRepo.__init__(self, url, **kwargs)
 
-        self.remotes = remotes
+        if kwargs.get('remotes') is not None:
+            warnings.warn(
+                "'remotes' is deprecated and will be removed in 0.5.",
+                DeprecationWarning,
+                stacklevel=2,
+            )
+        BaseRepo.__init__(self, url, **kwargs)
 
     def get_revision(self):
         """Return current revision. Initial repositories return 'initial'."""
@@ -106,7 +169,13 @@ class GitRepo(BaseRepo):
         return url, rev
 
     def obtain(self):
-        """Retrieve the repository, clone if doesn't exist."""
+        """Retrieve the repository, clone if doesn't exist.
+
+        .. versionchanged:: 0.4.0
+
+           No longer sets remotes. This is now done manually through 
+           :meth:`~.set_remote`.
+        """
         self.check_destination()
 
         url = self.url
@@ -121,11 +190,6 @@ class GitRepo(BaseRepo):
         self.info('Cloning.')
         self.run(cmd, log_in_real_time=True)
 
-        if self.remotes:
-            for r in self.remotes:
-                self.error('Adding remote %s <%s>' % (r['remote_name'], r['url']))
-                self.remote_set(name=r['remote_name'], url=r['url'], overwrite=True)
-
         self.info('Initializing submodules.')
         self.run(['submodule', 'init'], log_in_real_time=True)
         cmd = ['submodule', 'update', '--recursive', '--init']
@@ -172,7 +236,8 @@ class GitRepo(BaseRepo):
 
         # show-ref output is in the form "<sha> refs/remotes/<remote>/<tag>"
         # we must strip the remote from the tag.
-        git_remote_name = self.git_remote_name
+        git_remote_name = self.get_current_remote_name()
+
         if "refs/remotes/%s" % git_tag in show_ref_output:
             m = re.match(
                 r'^[0-9a-f]{40} refs/remotes/'
@@ -278,11 +343,28 @@ class GitRepo(BaseRepo):
         cmd.extend(self.git_submodules)
         self.run(cmd, log_in_real_time=True)
 
-    @property
-    def remotes_get(self):
+    def remotes(self, flat=False):
         """Return remotes like git remote -v.
 
-        :rtype: dict of tuples
+        :param flat: Return a dict of ``tuple`` instead of ``dict``. Default False.
+        :type flat: bool
+
+        .. versionchanged:: 0.4.0
+
+           Has been changed from property to method
+
+        .. versionchanged:: 0.4.0
+
+           The ``flat`` argument has been added to return remotes in ``tuple`` form
+
+        .. versionchanged:: 0.4.0
+
+           This used to return a dict of tuples, it now returns a dict of dictionaries
+           with ``name``, ``fetch_url``, and ``push_url``.
+
+        Returns
+        -------
+        dict
         """
         remotes = {}
 
@@ -290,56 +372,134 @@ class GitRepo(BaseRepo):
         ret = filter(None, cmd.split('\n'))
 
         for remote_name in ret:
-            remotes[remote_name] = self.remote_get(remote_name)
+            remotes[remote_name] = (
+                self.remote(remote_name) if flat else self.remote(remote_name)._asdict()
+            )
         return remotes
 
-    def remote_get(self, remote='origin'):
+    @property
+    def remotes_get(self):
+        """
+        .. versionchanged:: 0.4.0
+
+           The ``remotes_get`` property is deprecated and will be removed in 0.5. It
+           has been renamed ``remotes()`` and changed from property to a method.
+        """
+        warnings.warn(
+            "'remotes_get' is deprecated and will be removed in 0.5. "
+            "Use 'remotes()' method instead.",
+            DeprecationWarning,
+            stacklevel=2,
+        )
+
+        return self.remotes()
+
+    def remote(self, name, **kwargs):
         """Get the fetch and push URL for a specified remote name.
 
-        :param remote: the remote name used to define the fetch and push URL
-        :type remote: str
-        :returns: remote name and url in tuple form
-        :rtype: tuple
+        Parameters
+        ----------
+        name : str
+            The remote name used to define the fetch and push URL
+
+        Returns
+        -------
+        :class:`libvcs.git.GitRemote` : 
+            Remote name and url in tuple form
+
+        .. versionchanged:: 0.4.0
+
+           The ``remote`` argument was renamed to ``name`` and will be deprecated
+           in 0.5.
         """
+
+        if kwargs.get('remote') is not None:
+            warnings.warn(
+                "'remote' is deprecated and will be removed in 0.5. "
+                "Use 'name' instead.",
+                DeprecationWarning,
+                stacklevel=2,
+            )
+            name = kwargs.get('remote')
+
         try:
-            ret = self.run(['remote', 'show', '-n', remote])
+            ret = self.run(['remote', 'show', '-n', name])
             lines = ret.split('\n')
             remote_fetch_url = lines[1].replace('Fetch URL: ', '').strip()
             remote_push_url = lines[2].replace('Push  URL: ', '').strip()
-            if remote_fetch_url != remote and remote_push_url != remote:
-                res = (remote_fetch_url, remote_push_url)
-                return res
+            if remote_fetch_url != name and remote_push_url != name:
+                return GitRemote(
+                    name=name, fetch_url=remote_fetch_url, push_url=remote_push_url
+                )
             else:
                 return None
         except exc.LibVCSException:
             return None
 
-    def remote_set(self, url, name='origin', overwrite=False):
+    def remote_get(self, name='origin', **kwargs):
+        """
+        .. versionchanged:: 0.4.0
+
+           The ``remote_get`` method is deprecated and will be removed in 0.5.0. It has
+           been renamed ``remote`` 
+        """
+        warnings.warn(
+            "'remote_get' is deprecated and will be removed in 0.5. "
+            "Use 'remote' instead.",
+            DeprecationWarning,
+            stacklevel=2,
+        )
+
+        return self.remote(name=name, **kwargs)
+
+    def set_remote(self, name, url, overwrite=False):
         """Set remote with name and URL like git remote add.
 
-        :param url: defines the remote URL
-        :type url: str
         :param name: defines the remote name.
         :type name: str
+        :param url: defines the remote URL
+        :type url: str
+
+        .. versionadded:: 0.4.0
         """
 
         url = self.chomp_protocol(url)
 
-        if self.remote_get(name) and overwrite:
+        if self.remote(name) and overwrite:
             self.run(['remote', 'set-url', name, url])
         else:
             self.run(['remote', 'add', name, url])
-        return self.remote_get(remote=name)
+        return self.remote(name=name)
+
+    def remote_set(self, url, name='origin', overwrite=False, **kwargs):
+        """
+        .. versionchanged:: 0.4.0
+
+           The ``remote_set`` method is deprecated and will be removed in 0.5.0. It has
+           been renamed ``set_remote``.
+        """
+        warnings.warn(
+            "'remote_set' is deprecated and will be removed in 0.5. "
+            "Use 'set_remote' instead.",
+            DeprecationWarning,
+            stacklevel=2,
+        )
+
+        return self.set_remote(url=url, name=name, overwrite=overwrite, **kwargs)
 
     @staticmethod
     def chomp_protocol(url):
         """Return clean VCS url from RFC-style url
 
-        :param url: url
-        :type url: str
-        :rtype: str
-        :returns: url as VCS software would accept it
-        :seealso: #14
+        Parameters
+        ----------
+        url : str 
+            PIP-style url
+
+        Returns
+        -------
+        str :
+            URL as VCS software would accept it
         """
         if '+' in url:
             url = url.split('+', 1)[1]
@@ -355,3 +515,60 @@ class GitRepo(BaseRepo):
             url = url.replace('git+', 'git+ssh://')
             url = url.replace('ssh://', '')
         return url
+
+    def get_git_version(self):
+        """Return current version of git binary
+
+        Returns
+        -------
+        str
+        """
+        VERSION_PFX = 'git version '
+        version = self.run(['version'])
+        if version.startswith(VERSION_PFX):
+            version = version[len(VERSION_PFX) :].split()[0]
+        else:
+            version = ''
+        return '.'.join(version.split('.')[:3])
+
+    def status(self):
+        """Retrieve status of project in dict format.
+
+        Wraps ``git status --sb --porcelain=2``. Does not include changed files, yet.
+
+        Examples
+        --------
+
+        ::
+
+            print(git_repo.status())
+            {
+                "branch_oid": 'de6185fde0806e5c7754ca05676325a1ea4d6348',
+                "branch_head": 'fix-current-remote-name',
+                "branch_upstream": 'origin/fix-current-remote-name',
+                "branch_ab": '+0 -0',
+                "branch_ahead": '0',
+                "branch_behind": '0',
+            }
+
+        Returns
+        -------
+        dict :
+            Status of current checked out repository
+        """
+        return extract_status(self.run(['status', '-sb', '--porcelain=2']))
+
+    def get_current_remote_name(self):
+        """Retrieve name of the remote / upstream of currently checked out branch.
+
+        Returns
+        -------
+        str :
+            If upstream the same, returns ``branch_name``.
+            If upstream mismatches, returns ``remote_name/branch_name``.
+        """
+        match = self.status()
+
+        if match['branch_upstream'] is None:  # no upstream set
+            return match['branch_head']
+        return match['branch_upstream'].replace('/' + match['branch_head'], '')
